==> fusenshi-0.8.5.tgz: fusenshi-0.8.5/src/wnd/fusenshieditdialog.cpp <==
/*
 * $Id: fusenshieditdialog.cpp,v 1.10 2007/03/04 12:53:15 ozawa Exp $
 *
 * Copyright 2003- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.10 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "fusenshieditdialog.h"
#include "messagedialog.h"

#define TABS_WIDTH          8
#define MAX_UNDO_LEVELS     50

/**
 * 󥹥ȥ饯
 */
FusenshiEditDialog::FusenshiEditDialog(const FusenshiData* pData)
{
    if (!pData) {
        throw "Null Pointer Exception.";
    }
    
    int count = 0;
    
    m_pOriginalData = pData;
    m_cData = *pData;
    m_nResponse = 1;
    m_bRunning = true;
    
    set_wmclass(APP_NAME, "FusenshiEdit");
    set_title(_("Fusenshi - Edit"));
    set_modal(false);
    //set_position(Gtk::WIN_POS_CENTER);
    set_default_size(400, 300);
    property_destroy_with_parent() = true;

    // Gkt::SourceView
    Glib::RefPtr<Gtk::SourceBuffer> sourceBuffer = Gtk::SourceBuffer::create();
    m_cSourceView.set_buffer(sourceBuffer);
    m_cSourceView.set_left_margin(5);
    m_cSourceView.set_right_margin(5);
    m_cSourceView.set_cursor_visible(true);
    m_cSourceView.set_wrap_mode(pData->getWrapMode());
    m_cSourceView.set_justification(Gtk::JUSTIFY_LEFT);
    m_cSourceView.set_overwrite(false);
    m_cSourceView.set_accepts_tab(true);
    m_cSourceView.set_editable(true);
    m_cSourceView.set_show_line_numbers(false);
    m_cSourceView.set_show_line_markers(false);
    m_cSourceView.set_tabs_width(TABS_WIDTH);
    m_cSourceView.set_auto_indent(false);
    m_cSourceView.set_show_margin(false);
    sourceBuffer->set_max_undo_levels(MAX_UNDO_LEVELS);
    
    // Ƽ復åȤιۤ
    Gtk::Notebook* pNotebook = manage(new Gtk::Notebook());
    get_vbox()->pack_start(*pNotebook, true, true, 0);

    Gtk::ScrolledWindow* pScrolledWnd = manage(new Gtk::ScrolledWindow());
    pScrolledWnd->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    pNotebook->add(*pScrolledWnd);

    pScrolledWnd->add(m_cSourceView);

    Gtk::Fixed* pFixed = manage(new Gtk::Fixed());
    pNotebook->add(*pFixed);

    pNotebook->set_tab_label_text
        (*(pNotebook->get_nth_page(0)), _("Text"));
    pNotebook->set_tab_label_text
        (*(pNotebook->get_nth_page(1)), _("Property"));

    pFixed->put(m_cBackground, 10, 30);
    m_cBackground.set_size_request(200, 24);
    m_cBackground.set_max_length(20);

    pFixed->put(m_cForeground, 10, 80);
    m_cForeground.set_size_request(200, 24);
    m_cForeground.set_max_length(20);
    
    pFixed->put(m_cFontName, 10, 130);
    m_cFontName.set_size_request(200, 24);
    m_cFontName.set_max_length(256);
    
    pFixed->put(m_cWrapMode, 10, 180);
    m_cWrapMode.set_size_request(350, 30);
    for (count = 0; g_WndWrapModeTable[count].text; count++) {
        m_cWrapMode.append_text(_(g_WndWrapModeTable[count].text));
    }

    Gtk::Label* pLabel =
        manage(new Gtk::Label(_("Background Color:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 10);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    pLabel = manage(new Gtk::Label(_("Foreground Color:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 60);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    pLabel = manage(new Gtk::Label(_("Font:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 110);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);
    
    pLabel = manage(new Gtk::Label(_("Text wrap mode:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 160);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);
    
    Gtk::Button* pBgButton = manage(new Gtk::Button(_("Select..."), true));
    pFixed->put(*pBgButton, 220, 30);
    pBgButton->set_size_request(60, 24);

    Gtk::Button* pFgButton = manage(new Gtk::Button(_("Select..."), true));
    pFixed->put(*pFgButton, 220, 80);
    pFgButton->set_size_request(60, 24);

    Gtk::Button* pFnButton = manage(new Gtk::Button(_("Select..."), true));
    pFixed->put(*pFnButton, 220, 130);
    pFnButton->set_size_request(60, 24);

    get_action_area()->set_layout(Gtk::BUTTONBOX_END);

    Gtk::Button* pButton = manage(new Gtk::Button(Gtk::StockID("gtk-cancel")));
    add_action_widget(*pButton, Gtk::RESPONSE_CANCEL);
    //pButton->set_flags(Gtk::CAN_DEFAULT);

    pButton = manage(new Gtk::Button(Gtk::StockID("gtk-ok")));
    add_action_widget(*pButton, Gtk::RESPONSE_OK);
    pButton->set_flags(Gtk::CAN_DEFAULT);

    set_default_response(Gtk::RESPONSE_OK);

    pBgButton->signal_clicked().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_background_clicked));
    pFgButton->signal_clicked().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_foreground_clicked));
    pFnButton->signal_clicked().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_fontname_clicked));

    // ʲ¸Υǡ
    Glib::ustring buf;

    // ץѥƥ
    Gdk::Color bgColor((GdkColor*)pData->getBackground());
    color2string(bgColor.gobj(), buf);
    m_cBackground.set_text(buf);

    Gdk::Color fgColor((GdkColor*)pData->getForeground());
    color2string(fgColor.gobj(), buf);
    m_cForeground.set_text(buf);
    
    m_cFontName.set_text(pData->getFontName());

    // ʿطʿ
    m_cSourceView.modify_base(Gtk::STATE_NORMAL, bgColor);
    m_cSourceView.modify_text(Gtk::STATE_NORMAL, fgColor);
    
    // եȤ
    Pango::FontDescription* pFont = 
        new Pango::FontDescription(pData->getFontName());
    m_cSourceView.modify_font(*pFont);
    delete pFont;
    
    // ޤ֤⡼ɤ
    m_cWrapMode.set_active(0);
    for (count = 0; g_WndWrapModeTable[count].text; count++) {
        if (g_WndWrapModeTable[count].value == pData->getWrapMode()) {
            m_cWrapMode.set_active(count);
            break;
        }
    }
    
    // ƥȤ
    sourceBuffer->set_text(pData->getText());
    sourceBuffer->set_modified(false);
    
    signal_response().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_dialog_response), false);
}

/**
 * ǥȥ饯
 */
FusenshiEditDialog::~FusenshiEditDialog()
{
}

/**
 * ɽޤ
 * 
 * @return λɡ
 */
gint FusenshiEditDialog::run()
{
    show_all_children();
    set_focus(m_cSourceView);
    m_bRunning = true;
    show();

    /*
     * ƤΥ٥Ȥ Gtk::Main::iteration() ǽ뤿ᡢ
     * ʲνϥåɥդǤ
     */
    while (m_bRunning) {
        usleep(10 * 1000 /* 10ms */);

        while (Gtk::Main::events_pending()) {
            Gtk::Main::iteration();
        }
    }
    
    return m_nResponse;
}

/**
 * 䵻ǡޤ
 * 
 * @return 䵻ǡ
 */
const FusenshiData& FusenshiEditDialog::getNewData()
{
    return m_cData;
}

/**
 * ǡѹ̵ͭåޤ
 * 
 * @return ѹ true ᤷޤ
 */
bool FusenshiEditDialog::isChanged()
{
    return (m_cData != *m_pOriginalData);
}

/**
 * ɥκ׵ޤ
 */
bool FusenshiEditDialog::on_delete_event(GdkEventAny* pEvent)
{
    Gtk::Dialog::on_response(Gtk::RESPONSE_CANCEL);

    return true;
}

/**
 * α
 */
bool FusenshiEditDialog::on_key_press_event(GdkEventKey* pEvent)
{
    // ESC̵뤷ޤ
    if (GDK_Escape == pEvent->keyval) {
        return false;
    }
    
    return Gtk::Dialog::on_key_press_event(pEvent);
}

/**
 * ꡼α
 */
bool FusenshiEditDialog::on_key_release_event(GdkEventKey* pEvent)
{
    // ESC̵뤷ޤ
    if (GDK_Escape == pEvent->keyval) {
        return false;
    }

    return Gtk::Dialog::on_key_release_event(pEvent);
}

/**
 * 쥹ݥ
 */
void FusenshiEditDialog::on_dialog_response(int nResponse)
{
    m_nResponse = nResponse;
    
    if (Gtk::RESPONSE_OK == nResponse) {
        // ͤΥå
        int rowNum;
        Gdk::Color bgColor;
        Gdk::Color fgColor;
        
        if (!bgColor.parse(m_cBackground.get_text())) {
        	msgWarning(_("Unable to parse background color."));
            signal_response().emission_stop();
            return;
        }
        if (!fgColor.parse(m_cForeground.get_text())) {
        	msgWarning(_("Unable to parse foreground color."));
            signal_response().emission_stop();
            return;
        }
        
        rowNum = m_cWrapMode.get_active_row_number();
        if (0 > rowNum) {
        	msgWarning(_("Please choose a text wrap mode."));
            signal_response().emission_stop();
            return;
        }
        
        m_cData.setBackground(bgColor.gobj());
        m_cData.setForeground(fgColor.gobj());
        m_cData.setFontName(m_cFontName.get_text().c_str());
        m_cData.setWrapMode(g_WndWrapModeTable[rowNum].value);
        m_cData.setText(m_cSourceView.get_buffer()->get_text().c_str());
    }
    
    m_bRunning = false;

    Gtk::Dialog::on_response(nResponse);
}

/**
 * طʿ
 */
void FusenshiEditDialog::on_background_clicked()
{
    Gtk::ColorSelectionDialog colordlg;
    
    colordlg.set_title(_("Background Color:"));
    colordlg.set_transient_for(*this);
    colordlg.get_help_button()->hide();

    Gdk::Color bgColor;
    bgColor.parse(m_cBackground.get_text());
    colordlg.get_colorsel()->set_current_color(bgColor);
    
    if (Gtk::RESPONSE_OK == colordlg.run()) {
        Gdk::Color newColor = colordlg.get_colorsel()->get_current_color();
        Glib::ustring buf;
        
        color2string(newColor.gobj(), buf);
        m_cBackground.set_text(buf);
    }
}

/**
 * ʿ
 */
void FusenshiEditDialog::on_foreground_clicked()
{
    Gtk::ColorSelectionDialog colordlg;
    
    colordlg.set_title(_("Foreground Color:"));
    colordlg.set_transient_for(*this);
    colordlg.get_help_button()->hide();

    Gdk::Color fgColor;
    fgColor.parse(m_cForeground.get_text());
    colordlg.get_colorsel()->set_current_color(fgColor);
    
    if (Gtk::RESPONSE_OK == colordlg.run()) {
        Gdk::Color newColor = colordlg.get_colorsel()->get_current_color();
        Glib::ustring buf;
        
        color2string(newColor.gobj(), buf);
        m_cForeground.set_text(buf);
    }
}

/**
 * եȤ
 */
void FusenshiEditDialog::on_fontname_clicked()
{
    Gtk::FontSelectionDialog fontdlg;
    
    fontdlg.set_title(_("Font:"));
    fontdlg.set_transient_for(*this);
    fontdlg.get_apply_button()->hide();
    
    fontdlg.get_font_selection()->set_font_name(m_cFontName.get_text());
    
    if (Gtk::RESPONSE_OK == fontdlg.run()) {
        m_cFontName.set_text(fontdlg.get_font_selection()->get_font_name());
    }
}

==> fusenshi-0.9.0.tgz: fusenshi-0.9.0/src/wnd/fusenshieditdialog.cpp <==
/*
 * $Id: fusenshieditdialog.cpp,v 1.11 2007/06/25 06:19:21 ozawa Exp $
 *
 * Copyright 2003- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.11 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "fusenshieditdialog.h"
#include "messagedialog.h"

#define TABS_WIDTH          8
#define MAX_UNDO_LEVELS     50

/**
 * 󥹥ȥ饯
 */
FusenshiEditDialog::FusenshiEditDialog(const FusenshiData* pData, int nPage)
{
    if (!pData) {
        throw "Null Pointer Exception.";
    }
    
    if (nPage >= (int)pData->getTitle().size()) {
    	throw "Index out of bounds.";
    }
    
    int count = 0;
    
    m_pOriginalData = pData;
    m_cData = *pData;
    m_nPage = nPage;
    m_nResponse = 1;
    m_bRunning = true;
    
    set_wmclass(APP_NAME, "FusenshiEdit");
    set_title(_("Fusenshi - Edit"));
    set_modal(false);
    //set_position(Gtk::WIN_POS_CENTER);
    set_default_size(400, 350);
    property_destroy_with_parent() = true;

    // Gkt::SourceView
    Glib::RefPtr<Gtk::SourceBuffer> sourceBuffer = Gtk::SourceBuffer::create();
    m_cSourceView.set_buffer(sourceBuffer);
    m_cSourceView.set_left_margin(5);
    m_cSourceView.set_right_margin(5);
    m_cSourceView.set_cursor_visible(true);
    m_cSourceView.set_wrap_mode(pData->getWrapMode());
    m_cSourceView.set_justification(Gtk::JUSTIFY_LEFT);
    m_cSourceView.set_overwrite(false);
    m_cSourceView.set_accepts_tab(true);
    m_cSourceView.set_editable(true);
    m_cSourceView.set_show_line_numbers(false);
    m_cSourceView.set_show_line_markers(false);
    m_cSourceView.set_tabs_width(TABS_WIDTH);
    m_cSourceView.set_auto_indent(false);
    m_cSourceView.set_show_margin(false);
    sourceBuffer->set_max_undo_levels(MAX_UNDO_LEVELS);
    
    // Ƽ復åȤιۤ
    Gtk::Notebook* pNotebook = manage(new Gtk::Notebook());
    get_vbox()->pack_start(*pNotebook, true, true, 0);

    Gtk::ScrolledWindow* pScrolledWnd = manage(new Gtk::ScrolledWindow());
    pScrolledWnd->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    pNotebook->add(*pScrolledWnd);

    pScrolledWnd->add(m_cSourceView);

    Gtk::Fixed* pFixed = manage(new Gtk::Fixed());
    pNotebook->add(*pFixed);

    pNotebook->set_tab_label_text
        (*(pNotebook->get_nth_page(0)), _("Text"));
    pNotebook->set_tab_label_text
        (*(pNotebook->get_nth_page(1)), _("Property"));

	pFixed->put(m_cTitle, 10, 30);
	m_cTitle.set_size_request(200, 24);
    m_cTitle.set_max_length(MAX_TITLE_LEN);

    pFixed->put(m_cBackground, 10, 80);
    m_cBackground.set_size_request(200, 24);
    m_cBackground.set_max_length(20);

    pFixed->put(m_cForeground, 10, 130);
    m_cForeground.set_size_request(200, 24);
    m_cForeground.set_max_length(20);
    
    pFixed->put(m_cFontName, 10, 180);
    m_cFontName.set_size_request(200, 24);
    m_cFontName.set_max_length(256);
    
    pFixed->put(m_cWrapMode, 10, 230);
    m_cWrapMode.set_size_request(350, 30);
    for (count = 0; g_WndWrapModeTable[count].text; count++) {
        m_cWrapMode.append_text(_(g_WndWrapModeTable[count].text));
    }

    Gtk::Label* pLabel =
        manage(new Gtk::Label(_("Title:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 10);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    pLabel = manage(new Gtk::Label(_("Background Color:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 60);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    pLabel = manage(new Gtk::Label(_("Foreground Color:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 110);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    pLabel = manage(new Gtk::Label(_("Font:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 160);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);
    
    pLabel = manage(new Gtk::Label(_("Text wrap mode:"), 0.0, 0.5));
    pFixed->put(*pLabel, 10, 210);
    pLabel->set_size_request(150, 16);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);
    
    Gtk::Button* pBgButton = manage(new Gtk::Button(_("Select..."), true));
    pFixed->put(*pBgButton, 220, 80);
    pBgButton->set_size_request(60, 24);

    Gtk::Button* pFgButton = manage(new Gtk::Button(_("Select..."), true));
    pFixed->put(*pFgButton, 220, 130);
    pFgButton->set_size_request(60, 24);

    Gtk::Button* pFnButton = manage(new Gtk::Button(_("Select..."), true));
    pFixed->put(*pFnButton, 220, 180);
    pFnButton->set_size_request(60, 24);

    get_action_area()->set_layout(Gtk::BUTTONBOX_END);

    Gtk::Button* pButton = manage(new Gtk::Button(Gtk::StockID("gtk-cancel")));
    add_action_widget(*pButton, Gtk::RESPONSE_CANCEL);
    //pButton->set_flags(Gtk::CAN_DEFAULT);

    pButton = manage(new Gtk::Button(Gtk::StockID("gtk-ok")));
    add_action_widget(*pButton, Gtk::RESPONSE_OK);
    pButton->set_flags(Gtk::CAN_DEFAULT);

    set_default_response(Gtk::RESPONSE_OK);

    pBgButton->signal_clicked().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_background_clicked));
    pFgButton->signal_clicked().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_foreground_clicked));
    pFnButton->signal_clicked().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_fontname_clicked));

    // ʲ¸Υǡ
    Glib::ustring buf;

    // ץѥƥ
    Gdk::Color bgColor((GdkColor*)pData->getBackground());
    color2string(bgColor.gobj(), buf);
    m_cBackground.set_text(buf);

    Gdk::Color fgColor((GdkColor*)pData->getForeground());
    color2string(fgColor.gobj(), buf);
    m_cForeground.set_text(buf);
    
    m_cFontName.set_text(pData->getFontName());

    // ʿطʿ
    m_cSourceView.modify_base(Gtk::STATE_NORMAL, bgColor);
    m_cSourceView.modify_text(Gtk::STATE_NORMAL, fgColor);
    
    // եȤ
    Pango::FontDescription* pFont = 
        new Pango::FontDescription(pData->getFontName());
    m_cSourceView.modify_font(*pFont);
    delete pFont;
    
    // ޤ֤⡼ɤ
    m_cWrapMode.set_active(0);
    for (count = 0; g_WndWrapModeTable[count].text; count++) {
        if (g_WndWrapModeTable[count].value == pData->getWrapMode()) {
            m_cWrapMode.set_active(count);
            break;
        }
    }
    
    // ȥ
    m_cTitle.set_text(pData->getTitle().at(nPage));

    // ƥȤ
    sourceBuffer->set_text(pData->getText().at(nPage));
    sourceBuffer->set_modified(false);
    
    signal_response().connect
        (sigc::mem_fun(*this, &FusenshiEditDialog::on_dialog_response), false);
}

/**
 * ǥȥ饯
 */
FusenshiEditDialog::~FusenshiEditDialog()
{
}

/**
 * ɽޤ
 * 
 * @return λɡ
 */
gint FusenshiEditDialog::run()
{
    show_all_children();
    set_focus(m_cSourceView);
    m_bRunning = true;
    show();

    /*
     * ƤΥ٥Ȥ Gtk::Main::iteration() ǽ뤿ᡢ
     * ʲνϥåɥդǤ
     */
    while (m_bRunning) {
        usleep(10 * 1000 /* 10ms */);

        while (Gtk::Main::events_pending()) {
            Gtk::Main::iteration();
        }
    }
    
    return m_nResponse;
}

/**
 * 䵻ǡޤ
 * 
 * @return 䵻ǡ
 */
const FusenshiData& FusenshiEditDialog::getNewData()
{
    return m_cData;
}

/**
 * ǡѹ̵ͭåޤ
 * 
 * @return ѹ true ᤷޤ
 */
bool FusenshiEditDialog::isChanged()
{
    return (m_cData != *m_pOriginalData);
}

/**
 * ɥκ׵ޤ
 */
bool FusenshiEditDialog::on_delete_event(GdkEventAny* pEvent)
{
    Gtk::Dialog::on_response(Gtk::RESPONSE_CANCEL);

    return true;
}

/**
 * α
 */
bool FusenshiEditDialog::on_key_press_event(GdkEventKey* pEvent)
{
    // ESC̵뤷ޤ
    if (GDK_Escape == pEvent->keyval) {
        return false;
    }
    
    return Gtk::Dialog::on_key_press_event(pEvent);
}

/**
 * ꡼α
 */
bool FusenshiEditDialog::on_key_release_event(GdkEventKey* pEvent)
{
    // ESC̵뤷ޤ
    if (GDK_Escape == pEvent->keyval) {
        return false;
    }

    return Gtk::Dialog::on_key_release_event(pEvent);
}

/**
 * 쥹ݥ
 */
void FusenshiEditDialog::on_dialog_response(int nResponse)
{
    m_nResponse = nResponse;
    
    if (Gtk::RESPONSE_OK == nResponse) {
        // ͤΥå
        int rowNum;
        Gdk::Color bgColor;
        Gdk::Color fgColor;
        
        if (!bgColor.parse(m_cBackground.get_text())) {
        	msgWarning(_("Unable to parse background color."));
            signal_response().emission_stop();
            return;
        }
        if (!fgColor.parse(m_cForeground.get_text())) {
        	msgWarning(_("Unable to parse foreground color."));
            signal_response().emission_stop();
            return;
        }
        
        rowNum = m_cWrapMode.get_active_row_number();
        if (0 > rowNum) {
        	msgWarning(_("Please choose a text wrap mode."));
            signal_response().emission_stop();
            return;
        }
        
        m_cData.setBackground(bgColor.gobj());
        m_cData.setForeground(fgColor.gobj());
        m_cData.setFontName(m_cFontName.get_text().c_str());
        m_cData.setWrapMode(g_WndWrapModeTable[rowNum].value);
        
        StringArray title = m_cData.getTitle();
        StringArray text = m_cData.getText();
        title[m_nPage] = m_cTitle.get_text();
        text[m_nPage] = m_cSourceView.get_buffer()->get_text();
        m_cData.setText(title, text);
    }
    
    m_bRunning = false;

    Gtk::Dialog::on_response(nResponse);
}

/**
 * طʿ
 */
void FusenshiEditDialog::on_background_clicked()
{
    Gtk::ColorSelectionDialog colordlg;
    
    colordlg.set_title(_("Background Color:"));
    colordlg.set_transient_for(*this);
    colordlg.get_help_button()->hide();

    Gdk::Color bgColor;
    bgColor.parse(m_cBackground.get_text());
    colordlg.get_colorsel()->set_current_color(bgColor);
    
    if (Gtk::RESPONSE_OK == colordlg.run()) {
        Gdk::Color newColor = colordlg.get_colorsel()->get_current_color();
        Glib::ustring buf;
        
        color2string(newColor.gobj(), buf);
        m_cBackground.set_text(buf);
    }
}

/**
 * ʿ
 */
void FusenshiEditDialog::on_foreground_clicked()
{
    Gtk::ColorSelectionDialog colordlg;
    
    colordlg.set_title(_("Foreground Color:"));
    colordlg.set_transient_for(*this);
    colordlg.get_help_button()->hide();

    Gdk::Color fgColor;
    fgColor.parse(m_cForeground.get_text());
    colordlg.get_colorsel()->set_current_color(fgColor);
    
    if (Gtk::RESPONSE_OK == colordlg.run()) {
        Gdk::Color newColor = colordlg.get_colorsel()->get_current_color();
        Glib::ustring buf;
        
        color2string(newColor.gobj(), buf);
        m_cForeground.set_text(buf);
    }
}

/**
 * եȤ
 */
void FusenshiEditDialog::on_fontname_clicked()
{
    Gtk::FontSelectionDialog fontdlg;
    
    fontdlg.set_title(_("Font:"));
    fontdlg.set_transient_for(*this);
    fontdlg.get_apply_button()->hide();
    
    fontdlg.get_font_selection()->set_font_name(m_cFontName.get_text());
    
    if (Gtk::RESPONSE_OK == fontdlg.run()) {
        m_cFontName.set_text(fontdlg.get_font_selection()->get_font_name());
    }
}
